dagostinoTest <- function(x)
{
   DNAME <- deparse(substitute(x))
   x <- x[complete.cases(x)]
   n <- length(x)
   if (n<6) stop("sample size must be at least 6")
   meanX <- mean(x)
   s<- sqrt(mean((x-meanX)**2))
   a3 <- mean((x-meanX)**3)/s**3
   a4 <- mean((x-meanX)**4)/s**4
   SD3 <- sqrt(6*(n-2)/((n+1)*(n+3)))
   SD4 <- sqrt(24*(n-2)*(n-3)*n/((n+1)**2*(n+3)*(n+5)))
   U3 <- a3/SD3
   U4 <- (a4-3+6/(n+1))/SD4
   b <-(3*(n**2+27*n-70)*(n+1)*(n+3))/((n-2)*(n+5)*(n+7)*(n+9))
   W2 <- sqrt(2*(b-1))-1
   delta <- 1/sqrt(log(sqrt(W2)))
   a <- sqrt(2/(W2-1))
   Z3 <- delta*log((U3/a)+sqrt((U3/a)**2+1))
   B <- (6*(n*n-5*n+2)/((n+7)*(n+9)))*sqrt((6*(n+3)*(n+5))/(n*(n-2)*(n-3)))  
   A <- 6+(8/B)*((2/B)+sqrt(1+4/(B**2)))
   jm <- sqrt(2/(9*A))
   pos <- ((1-2/A)/(1+U4*sqrt(2/(A-4))))**(1/3)
   Z4 <- (1-2/(9*A)-pos)/jm
   omni <- Z3**2+Z4**2
   pZ3 <- 2*(1-pnorm(abs(Z3),0,1))
   pZ4 <- 2*(1-pnorm(abs(Z4),0,1))
   pomni <- 1-pchisq(omni,2)
   skewness <- c(Z3,pZ3)
   kurtosis <- c(Z4,pZ4)
   omnibus <- c(omni,pomni)
   DA <- cbind(skewness,kurtosis,omnibus)
   row.names(DA)<-c("statistics","p-value")
   return(DA)
}
skewness.test <- function(x)
{
   DNAME <- deparse(substitute(x))
   x <- x[complete.cases(x)]
   n <- length(x)
   if (n<8) stop("sample size must be at least 8")
   meanX <- mean(x)
   s<- sqrt(mean((x-meanX)**2))
   a3 <- mean((x-meanX)**3)/s**3
   SD3 <- sqrt(6*(n-2)/((n+1)*(n+3)))
   U3 <- a3/SD3
   b <-(3*(n**2+27*n-70)*(n+1)*(n+3))/((n-2)*(n+5)*(n+7)*(n+9))
   W2 <- sqrt(2*(b-1))-1
   delta <- 1/sqrt(log(sqrt(W2)))
   a <- sqrt(2/(W2-1))
   Z3 <- delta*log((U3/a)+sqrt((U3/a)**2+1))
   pZ3 <- 2*(1-pnorm(abs(Z3),0,1))
   names(Z3) <- "Z3"
   RVAL <- list(statistic=Z3,
        method="D'Agostino skewness normality test",
        p.value=pZ3,
        data.name=DNAME)
   class(RVAL) <- "htest"
   return(RVAL)
}
kurtosis.test <- function(x)
{
   DNAME <- deparse(substitute(x))
   x <- x[complete.cases(x)]
   n <- length(x)
   if (n<20) stop("sample size must be at least 20")
   meanX <- mean(x)
   s<- sqrt(mean((x-meanX)**2))
   a4 <- mean((x-meanX)**4)/s**4
   SD4 <- sqrt(24*(n-2)*(n-3)*n/((n+1)**2*(n+3)*(n+5)))
   U4 <- (a4-3+6/(n+1))/SD4
   B <- (6*(n*n-5*n+2)/((n+7)*(n+9)))*sqrt((6*(n+3)*(n+5))/(n*(n-2)*(n-3)))  
   A <- 6+(8/B)*((2/B)+sqrt(1+4/(B**2)))
   jm <- sqrt(2/(9*A))
   pos <- ((1-2/A)/(1+U4*sqrt(2/(A-4))))**(1/3)
   Z4 <- (1-2/(9*A)-pos)/jm
   pZ4 <- 2*(1-pnorm(abs(Z4),0,1))
   names(Z4) <- "Z4"
   RVAL <- list(statistic=Z4,
        method="D'Agostino kurtosis normality test",
        p.value=pZ4,
        data.name=DNAME)
   class(RVAL) <- "htest"
   return(RVAL)
}
omnibus.test <- function(x)
{
   DNAME <- deparse(substitute(x))
   x <- x[complete.cases(x)]
   n <- length(x)
   if (n<20) stop("sample size must be at least 20")
   meanX <- mean(x)
   s<- sqrt(mean((x-meanX)**2))
   a3 <- mean((x-meanX)**3)/s**3
   a4 <- mean((x-meanX)**4)/s**4
   SD3 <- sqrt(6*(n-2)/((n+1)*(n+3)))
   SD4 <- sqrt(24*(n-2)*(n-3)*n/((n+1)**2*(n+3)*(n+5)))
   U3 <- a3/SD3
   U4 <- (a4-3+6/(n+1))/SD4
   b <-(3*(n**2+27*n-70)*(n+1)*(n+3))/((n-2)*(n+5)*(n+7)*(n+9))
   W2 <- sqrt(2*(b-1))-1
   delta <- 1/sqrt(log(sqrt(W2)))
   a <- sqrt(2/(W2-1))
   Z3 <- delta*log((U3/a)+sqrt((U3/a)**2+1))
   B <- (6*(n*n-5*n+2)/((n+7)*(n+9)))*sqrt((6*(n+3)*(n+5))/(n*(n-2)*(n-3)))  
   A <- 6+(8/B)*((2/B)+sqrt(1+4/(B**2)))
   jm <- sqrt(2/(9*A))
   pos <- ((1-2/A)/(1+U4*sqrt(2/(A-4))))**(1/3)
   Z4 <- (1-2/(9*A)-pos)/jm
   omni <- Z3**2+Z4**2
   pomni <- 1-pchisq(omni,2)
   df <- c(2)
   names(omni) <- "Chi2"
   names(df) <- "df"
   RVAL <- list(statistic=omni,
        method="D'Agostino omnibus normality test",
        parameter=df,
        p.value=pomni,
        data.name=DNAME)
   class(RVAL) <- "htest"
   return(RVAL)
}

